@@ -166,5 +166,5 @@ def device_classes():
 
     for config in apps.get_app_configs():
         for model in config.get_models():
-            if issubclass(model, Device):
+            if issubclass(model, Device) and not model._meta.proxy:
                 yield model
