@@ -102,6 +102,7 @@ class HOTPDeviceAdmin(admin.ModelAdmin):
     # Columns
     #
 
+    @admin.display(description="QR Code")
     def qrcode_link(self, device):
         try:
             href = reverse('admin:otp_hotp_hotpdevice_config', kwargs={'pk': device.pk})
@@ -111,8 +112,6 @@ class HOTPDeviceAdmin(admin.ModelAdmin):
 
         return link
 
-    qrcode_link.short_description = "QR Code"
-
     #
     # Custom views
     #
