@@ -1,24 +1,16 @@
-from datetime import timedelta
 from doctest import DocTestSuite
 from io import StringIO
 import pickle
 from threading import Thread
 import unittest
 
-from freezegun import freeze_time
-
-from django.contrib.auth import get_user_model
 from django.contrib.auth.models import AnonymousUser
 from django.core.management import call_command
 from django.core.management.base import CommandError
 from django.db import IntegrityError, connection
-from django.test import RequestFactory
-from django.test import TestCase as DjangoTestCase
-from django.test import TransactionTestCase as DjangoTransactionTestCase
-from django.test import skipUnlessDBFeature
+from django.test import AsyncRequestFactory, RequestFactory, skipUnlessDBFeature
 from django.test.utils import override_settings
 from django.urls import reverse
-from django.utils import timezone
 
 from django_otp import (
     DEVICE_ID_SESSION_KEY,
@@ -31,9 +23,10 @@ from django_otp import (
 )
 from django_otp.forms import OTPTokenForm, otp_verification_failed
 from django_otp.middleware import OTPMiddleware
-from django_otp.models import GenerateNotAllowed, VerifyNotAllowed
 from django_otp.plugins.otp_static.models import StaticDevice, StaticToken
 
+from .test_utils import TestCase, TransactionTestCase
+
 
 def load_tests(loader, tests, pattern):
     suite = unittest.TestSuite()
@@ -53,309 +46,6 @@ class TestThread(Thread):
         connection.close()
 
 
-class OTPTestCaseMixin:
-    """
-    Utilities for dealing with custom user models.
-    """
-
-    @classmethod
-    def setUpClass(cls):
-        super().setUpClass()
-
-        cls.User = get_user_model()
-        cls.USERNAME_FIELD = cls.User.USERNAME_FIELD
-
-    def create_user(self, username, password, **kwargs):
-        """
-        Try to create a user, honoring the custom user model, if any.
-
-        This may raise an exception if the user model is too exotic for our
-        purposes.
-        """
-        return self.User.objects.create_user(username, password=password, **kwargs)
-
-
-class TestCase(OTPTestCaseMixin, DjangoTestCase):
-    pass
-
-
-class TransactionTestCase(OTPTestCaseMixin, DjangoTransactionTestCase):
-    pass
-
-
-class TimestampTestMixin:
-    """
-    Generic tests for :class:`~django_otp.models.TimestampMixin`.
-
-    Implementing tests must initialize `self.device` with the model instance to
-    test and provide `valid_token` and `invalid_token` methods for verifying
-    token behavior.
-
-    Includes tests to:
-
-    - Check automatic setting of `created_at` upon object creation.
-    - Validate that `last_used_at` is initially None and updated only after
-      successful token verification.
-    - Ensure `set_last_used_timestamp` behaves correctly, respecting the
-      `commit` parameter.
-
-    """
-
-    def setUp(self):
-        self.device = None
-
-    def valid_token(self):
-        """Returns a valid token to pass to our device under test."""
-        raise NotImplementedError()
-
-    def invalid_token(self):
-        """Returns an invalid token to pass to our device under test."""
-        raise NotImplementedError()
-
-    #
-    # Tests
-    #
-
-    def test_created_at_set_on_creation(self):
-        """Verify that the `created_at` field is automatically set upon creation."""
-        self.assertIsNotNone(
-            self.device.created_at, "created_at should be automatically set."
-        )
-
-    def test_last_used_at_initially_none(self):
-        """Ensure `last_used_at` is None upon initial creation."""
-        self.assertIsNone(
-            self.device.last_used_at, "last_used_at should be None initially."
-        )
-
-    def test_set_last_used_timestamp_updates_field(self):
-        """Check if `set_last_used_timestamp` correctly updates the `last_used_at` field."""
-        self.device.set_last_used_timestamp(commit=True)
-        self.device.refresh_from_db()  # Assuming it's a persisted model
-
-        self.assertIsNotNone(
-            self.device.last_used_at, "last_used_at should be updated."
-        )
-
-    def test_set_last_used_timestamp_without_commit(self):
-        """
-        Ensure `set_last_used_timestamp` updates `last_used_at` without persisting
-        when commit=False.
-        """
-        original_last_used_at = self.device.last_used_at
-        self.device.set_last_used_timestamp(commit=False)
-        # Check in-memory update without saving
-        self.assertNotEqual(
-            self.device.last_used_at,
-            original_last_used_at,
-            "last_used_at should be updated in memory without commit.",
-        )
-
-        # Refresh from db to confirm it wasn't committed
-        self.device.refresh_from_db()
-        self.assertEqual(
-            self.device.last_used_at,
-            original_last_used_at,
-            "last_used_at should not be updated in db without commit.",
-        )
-
-    def test_verify_token_successful_updates_last_used_at(self):
-        """
-        Verifying with a valid token updates 'last_used_at'.
-        """
-        valid_token = self.valid_token()  # Method to generate a valid token
-        initial_last_used_at = self.device.last_used_at
-        verified = self.device.verify_token(valid_token)
-
-        self.assertTrue(verified, "Token should be verified successfully.")
-        self.device.refresh_from_db()
-        self.assertNotEqual(
-            self.device.last_used_at,
-            initial_last_used_at,
-            "'last_used_at' should be updated on successful verification.",
-        )
-
-    def test_verify_token_failed_does_not_update_last_used_at(self):
-        """
-        Verifying with an invalid token does not update 'last_used_at'.
-        """
-        invalid_token = self.invalid_token()  # Method to generate an invalid token
-        initial_last_used_at = self.device.last_used_at
-        verified = self.device.verify_token(invalid_token)
-
-        self.assertFalse(verified, "Token should not be verified.")
-        self.device.refresh_from_db()
-        self.assertEqual(
-            self.device.last_used_at,
-            initial_last_used_at,
-            "'last_used_at' should not be updated on failed verification.",
-        )
-
-
-class ThrottlingTestMixin:
-    """
-    Generic tests for throttled devices.
-
-    Any concrete device implementation that uses throttling should define a
-    TestCase subclass that includes this as a base class. This will help verify
-    a correct integration of ThrottlingMixin.
-
-    Subclasses are responsible for populating self.device with a device to test
-    as well as implementing methods to generate tokens to test with.
-
-    """
-
-    def setUp(self):
-        self.device = None
-
-    def valid_token(self):
-        """Returns a valid token to pass to our device under test."""
-        raise NotImplementedError()
-
-    def invalid_token(self):
-        """Returns an invalid token to pass to our device under test."""
-        raise NotImplementedError()
-
-    #
-    # Tests
-    #
-
-    def test_delay_imposed_after_fail(self):
-        verified1 = self.device.verify_token(self.invalid_token())
-        self.assertFalse(verified1)
-        verified2 = self.device.verify_token(self.valid_token())
-        self.assertFalse(verified2)
-
-    def test_delay_after_fail_expires(self):
-        verified1 = self.device.verify_token(self.invalid_token())
-        self.assertFalse(verified1)
-        with freeze_time() as frozen_time:
-            # With default settings initial delay is 1 second
-            frozen_time.tick(delta=timedelta(seconds=1.1))
-            verified2 = self.device.verify_token(self.valid_token())
-            self.assertTrue(verified2)
-
-    def test_throttling_failure_count(self):
-        self.assertEqual(self.device.throttling_failure_count, 0)
-        for i in range(0, 5):
-            self.device.verify_token(self.invalid_token())
-            # Only the first attempt will increase throttling_failure_count,
-            # the others will all be within 1 second of first
-            # and therefore not count as attempts.
-            self.assertEqual(self.device.throttling_failure_count, 1)
-
-    def test_verify_is_allowed(self):
-        # Initially should be allowed
-        verify_is_allowed1, data1 = self.device.verify_is_allowed()
-        self.assertEqual(verify_is_allowed1, True)
-        self.assertEqual(data1, None)
-
-        # After failure, verify is not allowed
-        with freeze_time():
-            self.device.verify_token(self.invalid_token())
-            verify_is_allowed2, data2 = self.device.verify_is_allowed()
-            self.assertEqual(verify_is_allowed2, False)
-            self.assertEqual(
-                data2,
-                {
-                    'reason': VerifyNotAllowed.N_FAILED_ATTEMPTS,
-                    'failure_count': 1,
-                    'locked_until': timezone.now() + timezone.timedelta(seconds=1),
-                },
-            )
-
-        # After a successful attempt, should be allowed again
-        with freeze_time() as frozen_time:
-            frozen_time.tick(delta=timedelta(seconds=1.1))
-            self.device.verify_token(self.valid_token())
-
-            verify_is_allowed3, data3 = self.device.verify_is_allowed()
-            self.assertEqual(verify_is_allowed3, True)
-            self.assertEqual(data3, None)
-
-
-class CooldownTestMixin:
-    def setUp(self):
-        self.device = None
-
-    def valid_token(self):
-        """Returns a valid token to pass to our device under test."""
-        raise NotImplementedError()
-
-    def invalid_token(self):
-        """Returns an invalid token to pass to our device under test."""
-        raise NotImplementedError()
-
-    #
-    # Tests
-    #
-
-    def test_generate_is_allowed_on_first_try(self):
-        """Token generation should be allowed on first try."""
-        allowed, _ = self.device.generate_is_allowed()
-        self.assertTrue(allowed)
-
-    def test_cooldown_imposed_after_successful_generation(self):
-        """
-        Token generation before cooldown should not be allowed
-        and the relevant reason should be returned.
-        """
-        with freeze_time():
-            self.device.generate_challenge()
-            self.device.refresh_from_db()
-            allowed, details = self.device.generate_is_allowed()
-
-            self.assertFalse(allowed)
-            self.assertEqual(
-                details['reason'], GenerateNotAllowed.COOLDOWN_DURATION_PENDING
-            )
-
-    def test_cooldown_expire_time(self):
-        """
-        When token generation is not allowed, the cooldown expire time
-        should be returned.
-        """
-        with freeze_time():
-            self.device.generate_challenge()
-            self.device.refresh_from_db()
-            _, details = self.device.generate_is_allowed()
-            self.assertEqual(
-                details['next_generation_at'], timezone.now() + timedelta(seconds=10)
-            )
-
-    def test_cooldown_reset(self):
-        """Cooldown can be reset and allow token generation again before the initial period expires."""
-        with freeze_time():
-            self.device.generate_is_allowed()
-            self.device.refresh_from_db()
-            self.device.cooldown_reset()
-            self.device.refresh_from_db()
-            allowed, _ = self.device.generate_is_allowed()
-            self.assertTrue(allowed)
-
-    def test_valid_token_verification_resets_cooldown(self):
-        """When the token is verified, the cooldown period is reset."""
-        with freeze_time():
-            self.device.generate_challenge()
-            self.device.refresh_from_db()
-            verified = self.device.verify_token(self.valid_token())
-            self.assertTrue(verified)
-            self.device.refresh_from_db()
-            allowed, _ = self.device.generate_is_allowed()
-            self.assertTrue(allowed)
-
-    def test_invalid_token_verification_does_not_reset_cooldown(self):
-        """When the token is not verified, the cooldown period is not reset."""
-        with freeze_time():
-            self.device.generate_challenge()
-            self.device.refresh_from_db()
-            verified = self.device.verify_token(self.invalid_token())
-            self.assertFalse(verified)
-            self.device.refresh_from_db()
-            allowed, _ = self.device.generate_is_allowed()
-            self.assertFalse(allowed)
-
-
 @override_settings(OTP_STATIC_THROTTLE_FACTOR=0)
 class APITestCase(TestCase):
     def setUp(self):
@@ -521,6 +211,104 @@ class OTPMiddlewareTestCase(TestCase):
         pickle.dumps(request.user)
 
 
+class OTPMiddlewareAsyncTestCase(TestCase):
+    def setUp(self):
+        self.factory = AsyncRequestFactory()
+
+        try:
+            self.alice = self.create_user("alice", "password")
+            self.bob = self.create_user("bob", "password")
+        except IntegrityError:
+            self.skipTest("Unable to create a test user.")
+        else:
+            for user in (self.alice, self.bob):
+                device = user.staticdevice_set.create()
+                device.token_set.create(token=user.get_username())
+
+        # Precompute anything that would otherwise hit the (sync) ORM inside async tests.
+        alice_device = self.alice.staticdevice_set.get()
+        bob_device = self.bob.staticdevice_set.get()
+
+        self.alice_device_pid = alice_device.persistent_id
+        self.bob_device_pid = bob_device.persistent_id
+        self.alice_device_legacy_pid = "{}.{}/{}".format(
+            alice_device.__module__, alice_device.__class__.__name__, alice_device.id
+        )
+
+        async def get_response(request):
+            return None
+
+        self.middleware = OTPMiddleware(get_response)
+
+    async def _run_middleware(self, user, session):
+        request = self.factory.get("/")
+        request.session = session
+        request.user = user
+
+        async def auser():
+            return user
+
+        request.auser = auser
+
+        await self.middleware(request)
+        return request
+
+    async def test_verified(self):
+        request = await self._run_middleware(
+            self.alice, {DEVICE_ID_SESSION_KEY: self.alice_device_pid}
+        )
+
+        user = await request.auser()
+        self.assertTrue(user.is_verified())
+
+    async def test_verified_legacy_device_id(self):
+        request = await self._run_middleware(
+            self.alice, {DEVICE_ID_SESSION_KEY: self.alice_device_legacy_pid}
+        )
+
+        user = await request.auser()
+        self.assertTrue(user.is_verified())
+
+    async def test_unverified(self):
+        request = await self._run_middleware(self.alice, {})
+
+        user = await request.auser()
+        self.assertFalse(user.is_verified())
+
+    async def test_no_device(self):
+        request = await self._run_middleware(
+            self.alice, {DEVICE_ID_SESSION_KEY: "otp_static.staticdevice/0"}
+        )
+
+        user = await request.auser()
+        self.assertFalse(user.is_verified())
+
+    async def test_no_model(self):
+        request = await self._run_middleware(
+            self.alice, {DEVICE_ID_SESSION_KEY: "otp_bogus.bogusdevice/0"}
+        )
+
+        user = await request.auser()
+        self.assertFalse(user.is_verified())
+
+    async def test_wrong_user(self):
+        request = await self._run_middleware(
+            self.alice, {DEVICE_ID_SESSION_KEY: self.bob_device_pid}
+        )
+
+        user = await request.auser()
+        self.assertFalse(user.is_verified())
+
+    async def test_pickling(self):
+        request = await self._run_middleware(
+            self.alice, {DEVICE_ID_SESSION_KEY: self.alice_device_pid}
+        )
+
+        user = await request.auser()
+        # Should not raise an exception.
+        pickle.dumps(user)
+
+
 class LoginViewTestCase(TestCase):
     def setUp(self):
         try:
